@@ -46,7 +46,7 @@ RunModel_GR4J <- function(InputsModel,RunOptions,Param){
       }
 
     ##Call_fortan
-      RESULTS <- .Fortran("frun_GR4J",PACKAGE="airGR",
+      RESULTS <- .Fortran("frun_gr4j",PACKAGE="airGR",
                  ##inputs
                      LInputs=LInputSeries,                             ### length of input and output series
                      InputsPrecip=InputsModel$Precip[IndPeriod1],      ### input series of total precipitation [mm/d]
